"use strict";
// (C)opyright 2021-07-15 Dirk Holtwick, holtwick.it. All rights reserved.
Object.defineProperty(exports, "__esModule", { value: true });
exports.LoggerBrowserClassicHandler = void 0;
const time_1 = require("../common/time");
const log_1 = require("../common/log");
const log_colors_1 = require("./log-colors");
const log_filter_1 = require("../common/log-filter");
let namespaces = {};
let time = time_1.getTimestamp();
const useColors = log_colors_1.supportsColors();
/** @deprecated */
function LoggerBrowserClassicHandler(level = log_1.LogLevel.debug, opt = {
    colors: true,
    levelHelper: false,
    nameBrackets: true,
}) {
    const matches = log_filter_1.useNamespaceFilter(localStorage.debug);
    return (msg) => {
        if (msg.level < level)
            return;
        if (!matches(msg.name))
            return;
        const timeNow = time_1.getTimestamp();
        let name = msg.name || "";
        let ninfo = namespaces[name || ""];
        if (ninfo == null) {
            ninfo = {
                color: log_colors_1.selectColor(name),
                // time: timeNow
            };
            namespaces[name] = ninfo;
        }
        const diff = time_1.formatMilliseconds(timeNow - time);
        let args;
        if (opt.colors && useColors) {
            args = opt.nameBrackets ? [`%c[${name}]`] : [`%c${name}`];
            args.push(`color:${ninfo.color}`);
            args.push(...msg.messages);
        }
        else {
            args = [name, ...msg.messages];
        }
        args.push(`+${diff}`);
        switch (msg.level) {
            case log_1.LogLevel.info:
                if (opt.levelHelper)
                    args[0] = `I|*   ` + args[0];
                console.info(...args);
                break;
            case log_1.LogLevel.warn:
                if (opt.levelHelper)
                    args[0] = `W|**  ` + args[0];
                console.warn(...args);
                break;
            case log_1.LogLevel.error:
                if (opt.levelHelper)
                    args[0] = `E|*** ` + args[0];
                console.error(...args);
                break;
            default:
                if (opt.levelHelper)
                    args[0] = `D|    ` + args[0];
                console.debug(...args);
                break;
        }
    };
}
exports.LoggerBrowserClassicHandler = LoggerBrowserClassicHandler;
//# sourceMappingURL=log-browser-classic.js.map