"use strict";
// (C)opyright 2021-07-15 Dirk Holtwick, holtwick.it. All rights reserved.
Object.defineProperty(exports, "__esModule", { value: true });
exports.activateConsoleDebug = exports.LoggerBrowserHandler = void 0;
const time_js_1 = require("../common/time.js");
const log_1 = require("../common/log");
const log_colors_js_1 = require("./log-colors.js");
const log_filter_js_1 = require("../common/log-filter.js");
const deep_js_1 = require("../common/deep.js");
const styleFont = `font-family: "JetBrains Mono", Menlo; font-size: 11px;`;
const styleDefault = `${styleFont}`;
const styleBold = `font-weight: 600; ${styleFont}`;
const useColors = log_colors_js_1.supportsColors();
let namespaces = {};
let time = time_js_1.getTimestamp();
function LoggerBrowserHandler(opt = {}) {
    const { level = log_1.LogLevel.debug, colors = true, levelHelper = false, nameBrackets = true, padding = 16, } = opt;
    const matches = log_filter_js_1.useNamespaceFilter(localStorage.debug);
    return (msg) => {
        var _a, _b;
        if (msg.level < level)
            return;
        if (!matches(msg.name))
            return;
        const timeNow = time_js_1.getTimestamp();
        let name = msg.name || "";
        let ninfo = namespaces[name || ""];
        if (ninfo == null) {
            ninfo = {
                color: log_colors_js_1.selectColor(name),
                // time: timeNow
            };
            namespaces[name] = ninfo;
        }
        const diff = time_js_1.formatMilliseconds(timeNow - time);
        let args;
        if (padding > 0) {
            name = name.padEnd(16, " ");
        }
        if (colors && useColors) {
            args = [`%c${name}%c \t%s %c+${diff}`];
            args.push(`color:${ninfo.color}; ${styleBold}`);
            args.push(styleDefault);
            args.push((_b = (_a = msg.messages) === null || _a === void 0 ? void 0 : _a[0]) !== null && _b !== void 0 ? _b : "");
            args.push(`color:${ninfo.color};`);
            args.push(...msg.messages.slice(1));
        }
        else {
            args = [name, ...msg.messages, `+${diff}`];
        }
        // function consoleArgs(args: any[] = []): any[] {
        //   return [
        //     args
        //       .filter((a) => typeof a === "string")
        //       .map((a) => String(a))
        //       .join(" "),
        //     ...styles,
        //     ...args.filter((a) => typeof a !== "string"),
        //   ]
        // }
        switch (msg.level) {
            case log_1.LogLevel.info:
                if (opt.levelHelper)
                    args[0] = `I|*   ` + args[0];
                console.info(...args);
                break;
            case log_1.LogLevel.warn:
                if (opt.levelHelper)
                    args[0] = `W|**  ` + args[0];
                console.warn(...args);
                break;
            case log_1.LogLevel.error:
                if (opt.levelHelper)
                    args[0] = `E|*** ` + args[0];
                console.error(...args);
                break;
            default:
                if (opt.levelHelper)
                    args[0] = `D|    ` + args[0];
                console.debug(...args);
                break;
        }
    };
}
exports.LoggerBrowserHandler = LoggerBrowserHandler;
function LoggerBrowserSetupDebugFactory(opt = {}) {
    var _a, _b;
    const filter = (_b = (_a = opt.filter) !== null && _a !== void 0 ? _a : localStorage.zeed) !== null && _b !== void 0 ? _b : localStorage.debug;
    /// The trick is, that console called directly provides a reference to the source code.
    /// For the regular implementation this information is lost. But this approach has other
    /// drawbacks, therefore only use it in the Browser when actively debugging.
    return function LoggerBrowserDebugFactory(name = "") {
        let log;
        const matches = log_filter_js_1.useNamespaceFilter(filter);
        if (matches(name)) {
            let fixedArgs = [];
            if (useColors) {
                const color = log_colors_js_1.selectColor(name);
                fixedArgs.push(`%c${name.padEnd(16, " ")}%c \t%s`);
                fixedArgs.push(`color:${color}; ${styleBold}`);
                fixedArgs.push(styleDefault);
            }
            else {
                fixedArgs.push(`[${name}] \t%s`);
            }
            log = console.debug.bind(console, ...fixedArgs);
            log.debug = console.debug.bind(console, ...fixedArgs);
            log.info = console.info.bind(console, ...fixedArgs);
            log.warn = console.warn.bind(console, ...fixedArgs);
            log.error = console.error.bind(console, ...fixedArgs);
            log.assert = console.assert.bind(console);
            log.assertEqual = function (value, expected, ...args) {
                let equal = deep_js_1.deepEqual(value, expected);
                if (!equal) {
                    log.assert(equal, `Assert did fail. Expected ${expected} got ${value}`, expected, value, ...args);
                }
            };
            log.assertNotEqual = function (value, expected, ...args) {
                let equal = deep_js_1.deepEqual(value, expected);
                if (equal) {
                    log.assert(equal, `Assert did fail. Expected ${expected} not to be equal with ${value}`, expected, value, ...args);
                }
            };
        }
        else {
            const noop = () => { };
            log = noop;
            log.debug = noop;
            log.info = noop;
            log.warn = noop;
            log.error = noop;
            log.assert = noop;
            log.assertEqual = noop;
            log.assertNotEqual = noop;
        }
        log.extend = (subName) => {
            return LoggerBrowserDebugFactory(name ? `${name}:${subName}` : subName);
        };
        return log;
    };
}
function activateConsoleDebug(opt = {}) {
    log_1.Logger.setHandlers([LoggerBrowserHandler(opt)]); // Fallback for previously registered Loggers
    log_1.Logger.setFactory(LoggerBrowserSetupDebugFactory(opt));
}
exports.activateConsoleDebug = activateConsoleDebug;
// let klass = console
// let debug = console.debug.bind(window.console, klass.toString() + ": ")
// debug("test")
// console.debug("test2")
// let dd
// if (Function.prototype.bind) {
//   dd = Function.prototype.bind.call(console.log, console)
// } else {
//   dd = function () {
//     Function.prototype.apply.call(console.log, console, arguments)
//   }
// }
// dd("dd")
// let c = 1
// Object.defineProperty(window, "log2", {
//   get: () => {
//     return console.log.bind(
//       window.console,
//       "%c[log]%c %s" + c++,
//       "color:red",
//       ""
//     )
//   },
// })
// // usage:
// log2("Back to the future")
// log2("Back to the future")
// let plog = new Proxy(console.debug, {
//   apply: function (target, that, args) {
//     target.apply(that, args)
//     // base.apply(that, args);
//   },
// })
// let cons = console.debug
// let plog = (...args) => {
//   cons.apply(window.console, ["|", ...args])
// }
// plog("xxx")
// function a() {
//   var err = new Error()
//   var caller_line = err.stack.split("\n")[2]
//   var index = caller_line.indexOf("at ")
//   var clean = caller_line.slice(index + 2, caller_line.length)
//   clean = clean.replace(/\?t=\d+/, "").replace("@fs/", "")
//   console.log(clean)
//   console.log(
//     "http://localhost:8080/Users/dirk/work/viidoo/lib/src/browser/log-browser.ts:188:1 log-browser.ts:291:10"
//   )
// }
// function b() {
//   a()
// }
// b()
//# sourceMappingURL=log-browser.js.map