"use strict";
// (C)opyright 2021-07-15 Dirk Holtwick, holtwick.it. All rights reserved.
Object.defineProperty(exports, "__esModule", { value: true });
exports.arrayRandomElement = exports.arrayShuffleForce = exports.arrayShuffle = exports.arrayShuffleInPlace = exports.arrayIsEqual = exports.arraySortedNumbers = exports.arraySorted = exports.arrayEmptyInPlace = exports.arrayToggleInPlace = exports.arrayFilterInPlace = exports.arraySetElement = exports.arrayRemoveElement = exports.arraySymmetricDifference = exports.arrayIntersection = exports.arrayFlatten = exports.arrayUnion = exports.arrayMinus = exports.arrayUnique = void 0;
const orderby_js_1 = require("./orderby.js");
function arrayUnique(x) {
    return x.filter((n, index) => x.indexOf(n) === index);
}
exports.arrayUnique = arrayUnique;
function arrayMinus(x, y) {
    return arrayUnique(x.filter((n) => !y.includes(n)));
}
exports.arrayMinus = arrayMinus;
function arrayUnion(...a) {
    return arrayUnique(a.reduce((acc = [], value) => acc.concat(value), []));
}
exports.arrayUnion = arrayUnion;
/** `[1,[2,3]]` becomes `[1,2,3]` */
function arrayFlatten(list) {
    return list.reduce((a, b) => a.concat(Array.isArray(b) ? arrayFlatten(b) : b), []);
}
exports.arrayFlatten = arrayFlatten;
function arrayIntersection(x, y) {
    return arrayUnique(x).filter((n) => y.includes(n));
}
exports.arrayIntersection = arrayIntersection;
function arraySymmetricDifference(x, y) {
    return arrayMinus(arrayUnion(x, y), arrayIntersection(x, y));
    // return arrayUnique(x.filter(n => !y.includes(n)).concat(y.filter(n => !x.includes(n))))
}
exports.arraySymmetricDifference = arraySymmetricDifference;
// export function arrayApply<T>(fn:  any, a: T[]): T[] {
//   return a.reduce(fn, [])
// }
function arrayRemoveElement(arr, el) {
    if (arr && Array.isArray(arr)) {
        let index;
        while ((index = arr.indexOf(el)) !== -1) {
            // log("arrayRemoveElement remove", index, el)
            arr.splice(index, 1);
        }
        // log("arrayRemoveElement result", arr)
        return arr;
    }
    // log("arrayRemoveElement no array", arr, el)
    return [];
}
exports.arrayRemoveElement = arrayRemoveElement;
/** Only have it once in the set */
function arraySetElement(arr, el) {
    if (!arr.includes(el))
        arr.push(el);
    return arr;
}
exports.arraySetElement = arraySetElement;
// via https://stackoverflow.com/a/49587869 and Erwin
function arrayFilterInPlace(array, fn) {
    array.splice(0, array.length, ...array.filter(fn));
    return array;
}
exports.arrayFilterInPlace = arrayFilterInPlace;
// via https://stackoverflow.com/a/49587869 and Erwin
function arrayToggleInPlace(array, el) {
    const index = array.findIndex((e) => e === el);
    if (index >= 0)
        array.splice(index, 1);
    else
        array.push(el);
    return array;
}
exports.arrayToggleInPlace = arrayToggleInPlace;
function arrayEmptyInPlace(array) {
    array.splice(0, array.length);
    return array;
}
exports.arrayEmptyInPlace = arrayEmptyInPlace;
function arraySorted(arr, cond = orderby_js_1.cmp) {
    let copy = Array.from(arr);
    copy.sort(cond);
    return copy;
}
exports.arraySorted = arraySorted;
function arraySortedNumbers(arr) {
    return arraySorted(arr, (l, r) => l - r);
}
exports.arraySortedNumbers = arraySortedNumbers;
function arrayIsEqual(array1, array2) {
    return (array1.length === array2.length &&
        array1.every((value, index) => value === array2[index]));
}
exports.arrayIsEqual = arrayIsEqual;
function arrayShuffleInPlace(array) {
    array.sort(() => (Math.random() > 0.5 ? 1 : -1));
    return array;
}
exports.arrayShuffleInPlace = arrayShuffleInPlace;
function arrayShuffle(array) {
    return arrayShuffleInPlace(Array.from(array));
}
exports.arrayShuffle = arrayShuffle;
/** Randomly shuffle the order of the array's elements. Force to have a different order if array has more than one element. */
function arrayShuffleForce(array) {
    while (array.length > 1) {
        const copy = Array.from(array);
        arrayShuffleInPlace(copy);
        if (!arrayIsEqual(array, copy))
            return copy;
    }
    return array;
}
exports.arrayShuffleForce = arrayShuffleForce;
function arrayRandomElement(array) {
    return array[Math.floor(Math.random() * array.length)];
}
exports.arrayRandomElement = arrayRandomElement;
//# sourceMappingURL=array.js.map