import { Channel } from "./channel.js";
import { DefaultListener, EmitterHandler, ListenerSignature } from "./emitter.js";
import { SerialQueue } from "./queue.js";
export declare class Encoder {
    encode(data: any): string;
    decode(data: any): any;
}
export interface BridgeMessage {
    name: string;
    info: any;
    responseName?: string;
    success?: boolean;
    error?: string | null | undefined;
    requestPayload?: any;
}
interface BrideOptions {
    timeout?: number;
}
export declare class Bridge<L extends ListenerSignature<L> = DefaultListener> {
    channel: Channel;
    encoder: Encoder;
    queue: SerialQueue;
    subscribers: any;
    subscribersOnAny: any[];
    opt: BrideOptions;
    constructor(channel: Channel, opt?: BrideOptions);
    private _post;
    private _recv;
    emit<U extends keyof L>(event: U, ...args: Parameters<L[U]>): this;
    fetch<U extends keyof L>(event: U, ...args: Parameters<L[U]>): Promise<undefined | ReturnType<L[U]>>;
    onAny(fn: EmitterHandler): this;
    on<U extends keyof L>(event: U, listener: L[U]): {
        cleanup: () => void;
    };
    once<U extends keyof L>(event: U, listener: L[U]): this;
    off<U extends keyof L>(event: U, listener: L[U]): this;
    removeAllListeners(): this;
}
export {};
