"use strict";
// (C)opyright 2021-07-15 Dirk Holtwick, holtwick.it. All rights reserved.
Object.defineProperty(exports, "__esModule", { value: true });
exports.LoggerConsoleHandler = void 0;
const log_js_1 = require("./log.js");
/**
 * Very basic logger. Please take a look at the browser and node
 * optimized loggers. This one is just the absolute fallback option.
 *
 * @param level Log level
 * @returns Logger
 */
function LoggerConsoleHandler(opt = {}) {
    const { level = log_js_1.LogLevel.debug, colors = true, levelHelper = false, nameBrackets = true, padding = 16, filter = undefined, } = opt;
    return (msg) => {
        if (msg.level < level)
            return;
        let name = msg.name ? `[${msg.name}]` : "";
        switch (msg.level) {
            case log_js_1.LogLevel.info:
                console.info(`I|*   ${name}`, ...msg.messages);
                break;
            case log_js_1.LogLevel.warn:
                console.warn(`W|**  ${name}`, ...msg.messages);
                break;
            case log_js_1.LogLevel.error:
                console.error(`E|*** ${name}`, ...msg.messages);
                break;
            default:
                console.debug(`D|    ${name}`, ...msg.messages);
                break;
        }
    };
}
exports.LoggerConsoleHandler = LoggerConsoleHandler;
//# sourceMappingURL=log-console.js.map