"use strict";
// (C)opyright 2021-07-15 Dirk Holtwick, holtwick.it. All rights reserved.
Object.defineProperty(exports, "__esModule", { value: true });
exports.promisify = exports.isPromise = exports.waitOn = exports.tryTimeout = exports.isTimeout = exports.timoutError = exports.timeout = exports.timeoutReached = exports.immediate = exports.sleep = void 0;
const log_js_1 = require("./log.js");
const { warn } = log_js_1.Logger("zeed:promise");
/** Sleep for `milliSeconds`. Example 1s: `await sleep(1000)` */
async function sleep(milliSeconds) {
    return new Promise((resolve) => setTimeout(resolve, milliSeconds));
}
exports.sleep = sleep;
async function immediate() {
    return new Promise((resolve) => setTimeout(resolve, 0));
}
exports.immediate = immediate;
exports.timeoutReached = Symbol("timeout");
async function timeout(promise, milliSeconds, timeoutValue = exports.timeoutReached) {
    return new Promise(async (resolve, reject) => {
        let done = false;
        const timeout = setTimeout(() => {
            done = true;
            resolve(timeoutValue);
        }, milliSeconds);
        try {
            let result = await promise;
            clearTimeout(timeout);
            if (!done)
                resolve(result);
        }
        catch (err) {
            clearTimeout(timeout);
            if (!done)
                reject(err);
        }
    });
}
exports.timeout = timeout;
exports.timoutError = new Error("Timeout reached");
function isTimeout(value) {
    return value === exports.timeoutReached || value === exports.timoutError;
}
exports.isTimeout = isTimeout;
async function tryTimeout(promise, milliSeconds) {
    return new Promise(async (resolve, reject) => {
        let done = false;
        const timeout = setTimeout(() => {
            done = true;
            reject(exports.timoutError);
        }, milliSeconds);
        try {
            let result = await promise;
            clearTimeout(timeout);
            if (!done)
                resolve(result);
        }
        catch (err) {
            clearTimeout(timeout);
            if (!done)
                reject(err);
        }
    });
}
exports.tryTimeout = tryTimeout;
/** Wait for `event` on `obj` to emit. Resolve with result or reject on `timeout` */
function waitOn(obj, event, timeoutMS = 1000) {
    return new Promise((resolve, reject) => {
        let fn = (value) => {
            if (timer) {
                clearTimeout(timer);
                done();
                resolve(value);
            }
        };
        let done = () => {
            timer = null;
            if (obj.off) {
                obj.off(event, fn);
            }
            else if (obj.removeEventListener) {
                obj.removeEventListener(event, fn);
            }
            else {
                warn("No remove listener method found for", obj, event);
            }
        };
        let timer = setTimeout(() => {
            done();
            reject(new Error(`Did not response in time`));
        }, timeoutMS);
        if (obj.on) {
            obj.on(event, fn);
        }
        else if (obj.addEventListener) {
            obj.addEventListener(event, fn);
        }
        else {
            warn("No listener method found for", obj);
        }
    });
}
exports.waitOn = waitOn;
/** @deprecated */
function isPromise(value) {
    return Boolean(value &&
        (value instanceof Promise ||
            // @ts-ignore
            typeof value.then === "function"));
}
exports.isPromise = isPromise;
/** This is exactly what Prose.resolve(x) is supposed to be: return a Promise no matter what type x is */
function promisify(value) {
    // return isPromise(value) ? value : Promise.resolve(value)
    return Promise.resolve(value);
}
exports.promisify = promisify;
//# sourceMappingURL=promise.js.map