"use strict";
// (C)opyright 2021-07-15 Dirk Holtwick, holtwick.it. All rights reserved.
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileStorage = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const log_js_1 = require("../common/log.js");
const log = log_js_1.Logger("zeed:filestorage");
class FileStorage {
    constructor(basePath) {
        this.store = {};
        this.fileKeys = undefined;
        this.dirname = path_1.resolve(process.cwd(), basePath || ".fileStorage");
    }
    setItem(key, value) {
        const data = JSON.stringify(value);
        this.store[key] = data;
        try {
            const path = path_1.resolve(this.dirname, key + ".json");
            fs_1.mkdirSync(this.dirname, { recursive: true });
            fs_1.writeFileSync(path, data, "utf8");
        }
        catch (err) {
            log.error("setItem error", err);
        }
    }
    getItem(key) {
        if (this.store.hasOwnProperty(key)) {
            let value = this.store[key];
            return JSON.parse(value) || null;
        }
        else {
            try {
                const path = path_1.resolve(this.dirname, key + ".json");
                const data = fs_1.readFileSync(path, "utf8");
                if (data) {
                    return JSON.parse(data);
                }
            }
            catch (err) {
                log.error("getItem error", err);
            }
        }
        return null;
    }
    removeItem(key) {
        delete this.store[key];
        if (this.fileKeys != null) {
            const index = this.fileKeys.indexOf(key);
            if (index !== -1) {
                this.fileKeys.splice(index, 1);
            }
        }
        try {
            const path = path_1.resolve(this.dirname, key + ".json");
            fs_1.unlinkSync(path);
        }
        catch (err) { }
    }
    clear() {
        this.fileKeys = [];
        this.store = {};
        fs_1.rmSync(this.dirname, { recursive: true, force: true });
    }
    allKeys() {
        if (this.fileKeys == null) {
            try {
                this.fileKeys =
                    fs_1.readdirSync(this.dirname, { withFileTypes: true })
                        .filter((item) => !item.isDirectory() && item.name.endsWith(".json"))
                        .map((item) => item.name.slice(0, -5)) || [];
            }
            catch (err) { }
        }
        let keys = [...(this.fileKeys || [])];
        for (let key of Object.keys(this.store)) {
            if (!keys.includes(key)) {
                keys.push(key);
            }
        }
        keys.sort();
        return keys;
    }
}
exports.FileStorage = FileStorage;
//# sourceMappingURL=filestorage.js.map