"use strict";
// (C)opyright 2021-07-15 Dirk Holtwick, holtwick.it. All rights reserved.
Object.defineProperty(exports, "__esModule", { value: true });
exports.LoggerFileHandler = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const convert_js_1 = require("../common/convert.js");
const log_js_1 = require("../common/log.js");
let namespaces = {};
function LoggerFileHandler(path, opt = {}) {
    const { level = log_js_1.LogLevel.debug, colors = true, levelHelper = false, nameBrackets = true, padding = 16, filter = undefined, } = opt;
    path = path_1.resolve(process.cwd(), path);
    fs_1.mkdirSync(path_1.dirname(path), { recursive: true });
    var stream = fs_1.createWriteStream(path, { flags: "a" });
    // stream.end()
    return (msg) => {
        if (msg.level < level)
            return;
        const time = new Date().toISOString();
        let name = msg.name || "";
        let ninfo = namespaces[name || ""];
        if (ninfo == null) {
            namespaces[name] = ninfo;
        }
        let args = [
            `[${name || "*"}]`,
            convert_js_1.renderMessages(msg.messages, { pretty: false }),
        ];
        function write(...args) {
            stream.write(args.join("\t") + "\n");
        }
        switch (msg.level) {
            case log_js_1.LogLevel.info:
                write(time, `I|*  `, ...args);
                break;
            case log_js_1.LogLevel.warn:
                write(time, `W|** `, ...args);
                break;
            case log_js_1.LogLevel.error:
                write(time, `E|***`, ...args);
                break;
            default:
                write(time, `D|   `, ...args);
                break;
        }
    };
}
exports.LoggerFileHandler = LoggerFileHandler;
//# sourceMappingURL=log-file.js.map