// (C)opyright 2021-07-15 Dirk Holtwick, holtwick.it. All rights reserved.
import { cmp } from "./orderby.js";
export function arrayUnique(x) {
    return x.filter((n, index) => x.indexOf(n) === index);
}
export function arrayMinus(x, y) {
    return arrayUnique(x.filter((n) => !y.includes(n)));
}
export function arrayUnion(...a) {
    return arrayUnique(a.reduce((acc = [], value) => acc.concat(value), []));
}
/** `[1,[2,3]]` becomes `[1,2,3]` */
export function arrayFlatten(list) {
    return list.reduce((a, b) => a.concat(Array.isArray(b) ? arrayFlatten(b) : b), []);
}
export function arrayIntersection(x, y) {
    return arrayUnique(x).filter((n) => y.includes(n));
}
export function arraySymmetricDifference(x, y) {
    return arrayMinus(arrayUnion(x, y), arrayIntersection(x, y));
    // return arrayUnique(x.filter(n => !y.includes(n)).concat(y.filter(n => !x.includes(n))))
}
// export function arrayApply<T>(fn:  any, a: T[]): T[] {
//   return a.reduce(fn, [])
// }
export function arrayRemoveElement(arr, el) {
    if (arr && Array.isArray(arr)) {
        let index;
        while ((index = arr.indexOf(el)) !== -1) {
            // log("arrayRemoveElement remove", index, el)
            arr.splice(index, 1);
        }
        // log("arrayRemoveElement result", arr)
        return arr;
    }
    // log("arrayRemoveElement no array", arr, el)
    return [];
}
/** Only have it once in the set */
export function arraySetElement(arr, el) {
    if (!arr.includes(el))
        arr.push(el);
    return arr;
}
// via https://stackoverflow.com/a/49587869 and Erwin
export function arrayFilterInPlace(array, fn) {
    array.splice(0, array.length, ...array.filter(fn));
    return array;
}
// via https://stackoverflow.com/a/49587869 and Erwin
export function arrayToggleInPlace(array, el) {
    const index = array.findIndex((e) => e === el);
    if (index >= 0)
        array.splice(index, 1);
    else
        array.push(el);
    return array;
}
export function arrayEmptyInPlace(array) {
    array.splice(0, array.length);
    return array;
}
export function arraySorted(arr, cond = cmp) {
    let copy = Array.from(arr);
    copy.sort(cond);
    return copy;
}
export function arraySortedNumbers(arr) {
    return arraySorted(arr, (l, r) => l - r);
}
export function arrayIsEqual(array1, array2) {
    return (array1.length === array2.length &&
        array1.every((value, index) => value === array2[index]));
}
export function arrayShuffleInPlace(array) {
    array.sort(() => (Math.random() > 0.5 ? 1 : -1));
    return array;
}
export function arrayShuffle(array) {
    return arrayShuffleInPlace(Array.from(array));
}
/** Randomly shuffle the order of the array's elements. Force to have a different order if array has more than one element. */
export function arrayShuffleForce(array) {
    while (array.length > 1) {
        const copy = Array.from(array);
        arrayShuffleInPlace(copy);
        if (!arrayIsEqual(array, copy))
            return copy;
    }
    return array;
}
export function arrayRandomElement(array) {
    return array[Math.floor(Math.random() * array.length)];
}
//# sourceMappingURL=array.js.map