export declare enum LogLevel {
    debug = 0,
    info = 1,
    warn = 2,
    error = 3,
    fatal = 4,
    off = Infinity
}
export interface LogMessage {
    level: LogLevel;
    name: string;
    messages: any[];
    line?: number;
    file?: string;
    timestamp?: number;
}
export declare type LogHandler = (msg: LogMessage) => void;
export interface LoggerInterface {
    (...messages: any[]): void;
    active: boolean;
    level: LogLevel;
    debug(...messages: any[]): void;
    info(...messages: any[]): void;
    warn(...messages: any[]): void;
    error(...messages: any[]): void;
    assert(cond: any, ...messages: any[]): void;
    /** @deprecated use .assert */
    assertEqual(value: any, expected: any, ...args: any[]): void;
    /** @deprecated use .assert */
    assertNotEqual(value: any, expected: any, ...args: any[]): void;
    extend(prefix: string): LoggerInterface;
    factory?: LoggerContextInterface;
}
export interface LoggerContextInterface {
    (name?: string): LoggerInterface;
    registerHandler(handler: LogHandler): void;
    setFilter(namespaces: string): void;
    setHandlers(handlers?: LogHandler[]): void;
    setLock(lock: boolean): void;
    setLogLevel(level?: LogLevel): void;
    setFactory(factory: (name?: string) => LoggerInterface): void;
}
export interface LogHandlerOptions {
    level?: LogLevel;
    filter?: string;
    colors?: boolean;
    levelHelper?: boolean;
    nameBrackets?: boolean;
    padding?: number;
}
export declare function LoggerContext(prefix?: string): LoggerContextInterface;
declare global {
    interface ZeedGlobalContext {
        logger?: LoggerContextInterface;
    }
}
/** @deprecated Use `Logger` instead, it is global as well */
export declare const GlobalLogger: LoggerContextInterface;
export declare const Logger: LoggerContextInterface;
