// (C)opyright 2021-07-15 Dirk Holtwick, holtwick.it. All rights reserved.
import { readFileSync, writeFileSync, mkdirSync, rmSync, unlinkSync, readdirSync, } from "fs";
import { resolve } from "path";
import { Logger } from "../common/log.js";
const log = Logger("zeed:filestorage");
export class FileStorage {
    constructor(basePath) {
        this.store = {};
        this.fileKeys = undefined;
        this.dirname = resolve(process.cwd(), basePath || ".fileStorage");
    }
    setItem(key, value) {
        const data = JSON.stringify(value);
        this.store[key] = data;
        try {
            const path = resolve(this.dirname, key + ".json");
            mkdirSync(this.dirname, { recursive: true });
            writeFileSync(path, data, "utf8");
        }
        catch (err) {
            log.error("setItem error", err);
        }
    }
    getItem(key) {
        if (this.store.hasOwnProperty(key)) {
            let value = this.store[key];
            return JSON.parse(value) || null;
        }
        else {
            try {
                const path = resolve(this.dirname, key + ".json");
                const data = readFileSync(path, "utf8");
                if (data) {
                    return JSON.parse(data);
                }
            }
            catch (err) {
                log.error("getItem error", err);
            }
        }
        return null;
    }
    removeItem(key) {
        delete this.store[key];
        if (this.fileKeys != null) {
            const index = this.fileKeys.indexOf(key);
            if (index !== -1) {
                this.fileKeys.splice(index, 1);
            }
        }
        try {
            const path = resolve(this.dirname, key + ".json");
            unlinkSync(path);
        }
        catch (err) { }
    }
    clear() {
        this.fileKeys = [];
        this.store = {};
        rmSync(this.dirname, { recursive: true, force: true });
    }
    allKeys() {
        if (this.fileKeys == null) {
            try {
                this.fileKeys =
                    readdirSync(this.dirname, { withFileTypes: true })
                        .filter((item) => !item.isDirectory() && item.name.endsWith(".json"))
                        .map((item) => item.name.slice(0, -5)) || [];
            }
            catch (err) { }
        }
        let keys = [...(this.fileKeys || [])];
        for (let key of Object.keys(this.store)) {
            if (!keys.includes(key)) {
                keys.push(key);
            }
        }
        keys.sort();
        return keys;
    }
}
//# sourceMappingURL=filestorage.js.map